﻿using Funq;
using ServiceStack;
using ServiceStack.Mvc;
using System.IO;
using System.Text.RegularExpressions;
using System.Web.Mvc;
using Vintasoft.Data;
using Vintasoft.Imaging.Web.Services;
using Vintasoft.Shared.Web;

namespace WebImageViewerDemo.ServiceStack
{
    public class AppVintasoftHost : AppHostBase
    {
        public AppVintasoftHost()
            : base("MVC 5", typeof(VintasoftImageServiceStackService).Assembly)
        {
        }

        public override void Configure(Container container)
        {
            SetConfig(new HostConfig
            {
                HandlerFactoryPath = "Vintasoft",
                DefaultContentType = MimeTypes.Json
            });
            ControllerBuilder.Current.SetControllerFactory(
                new FunqControllerFactory(container));
        }
    }

    public class VintasoftImageServiceStackService : Service
    {

        #region Methods

        #region PUBLIC

        /// <summary>
        /// Returns an information about images from the specified files.
        /// </summary>
        /// <param name="fileUrl">Files URL on server.</param>
        /// <returns>Response, from server, which contains information
        /// about image collection.</returns>
        public virtual GetImageFilesInfoResponseParams Post(GetImageFileInfo requestParams)
        {
            VintasoftImageCollectionWebService service = CreateImageCollectionWebService(requestParams.sessionId);
            return service.GetImageFileInfo(requestParams);
        }

        /// <summary>
        /// Returns information about web images.
        /// </summary>
        /// <param name="requestParams">Information about web images.</param>
        /// <returns>Response from the server, which contains image infos.</returns>
        public virtual ImagePageInfoResponseParams[] Post(GetImagesInfo requestParams)
        {
            VintasoftImageWebService service = CreateImageWebService(requestParams.sessionId);
            return service.GetImagesInfo(requestParams);
        }
        
        /// <summary>
        /// Renders the image thumbnails.
        /// </summary>
        /// <param name="requestParams">Information about web image and
        /// parameters, which should be used for rendering the image thumbnails.</param>
        /// <returns>Response from the server, which contains information about images and
        /// image thumbnails.</returns>
        public virtual RenderThumbnailResponseParams[] Post(RenderThumbnails requestParams)
        {
            VintasoftImageWebService service = CreateImageWebService(requestParams.sessionId);
            RenderThumbnailResponseParams[] answer = service.RenderThumbnails(requestParams);
            CorrectImageFileResponseParams(answer);
            return answer;
        }
        
        /// <summary>
        /// Renders an image tile.
        /// </summary>
        /// <param name="requestParams">Information about web image and
        /// parameters, which should be used for rendering an image tile.</param>
        /// <returns>Response from the server, which contains information about web image and
        /// image tile.</returns>
        public virtual WebImageFileResponseParams Post(RenderImageTile requestParams)
        {
            VintasoftImageWebService service = CreateImageWebService(requestParams.sessionId);
            WebImageFileResponseParams answer = service.RenderImageTile(requestParams);
            CorrectImageFileResponseParams(answer);
            return answer;
        }

        #endregion


        #region PROTECTED

        #region Create WebService

        /// <summary>
        /// Creates the <see cref="VintasoftImageCollectionWebService"/>.
        /// </summary>
        protected VintasoftImageCollectionWebService CreateImageCollectionWebService(string sessionId)
        {
            IDataStorage storage = CreateDataStorage(sessionId);
            return new VintasoftImageCollectionWebService(storage);
        }

        /// <summary>
        /// Creates the <see cref="VintasoftImageWebService"/>.
        /// </summary>
        protected VintasoftImageWebService CreateImageWebService(string sessionId)
        {
            IDataStorage storage = CreateDataStorage(sessionId);
            ImageTilesCacheManager tilesCacheManager = CreateImageTilesCacheManager();
            ThumbnailsCacheManager thumbnailsCacheManager = CreateThumbnailsCacheManager();
            return new VintasoftImageWebService(storage, null, tilesCacheManager, thumbnailsCacheManager);
        }

        #endregion


        #region Create CacheManager

        /// <summary>
        /// Creates the ImageTilesCacheManager.
        /// </summary>
        /// <returns></returns>
        protected ImageTilesCacheManager CreateImageTilesCacheManager()
        {
            string workingCatalog = PathUtils.MapHostAbsolutePath("~/VintasoftCache/Tiles/");
            if (!Directory.Exists(workingCatalog))
                Directory.CreateDirectory(workingCatalog);
            return new ImageTilesCacheManager(workingCatalog);
        }

        /// <summary>
        /// Creates the ThumbnailsCacheManager.
        /// </summary>
        /// <returns></returns>
        protected ThumbnailsCacheManager CreateThumbnailsCacheManager()
        {
            string workingCatalog = PathUtils.MapHostAbsolutePath("~/VintasoftCache/Thumbnails/");
            if (!Directory.Exists(workingCatalog))
                Directory.CreateDirectory(workingCatalog);
            return new ThumbnailsCacheManager(workingCatalog);
        }

        #endregion


        /// <summary>
        /// Creates a data storage for current session.
        /// </summary>
        /// <param name="sessionId">The session identifier.</param>
        /// <returns>The data storage.</returns>
        protected IDataStorage CreateDataStorage(string sessionId)
        {
            string workingCatalog = PathUtils.MapHostAbsolutePath("~/Images/");
            if (!Directory.Exists(workingCatalog))
                Directory.CreateDirectory(workingCatalog);
            return new StreamDataStorageOnDisk(workingCatalog);
        }

        #endregion


        #region PRIVATE

        /// <summary>
        /// Corrects parameters in the image file responses.
        /// </summary>
        /// <param name="responses">An array that contains image file responses.</param>
        private void CorrectImageFileResponseParams(WebImageFileResponseParams[] responses)
        {
            for (int i = 0; i < responses.Length; i++)
                CorrectImageFileResponseParams(responses[i]);
        }

        /// <summary>
        /// Corrects parameters in the image file response.
        /// </summary>
        /// <param name="response">Image file responses.</param>
        private void CorrectImageFileResponseParams(WebImageFileResponseParams response)
        {
            if (response.success)
                response.fileId = ConvertRelativeUrlToAbsolute(response.fileId);
        }

        /// <summary>
        /// Converts the relative URL to the absolute URL.
        /// </summary>
        /// <param name="relativeUrl">Relative URL.</param>
        /// <returns>Absolute URL.</returns>
        private string ConvertRelativeUrlToAbsolute(string absolutePath)
        {
            string projectPath = PathUtils.MapHostAbsolutePath("~");
            if (!absolutePath.Contains(projectPath))
                return absolutePath;

            string relativePath = absolutePath.Replace(projectPath, "").Replace(@"\", "/");
            return HttpExtensions.ToAbsoluteUri(relativePath);
        }

        #endregion

        #endregion

    }
}